@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Foundation and Contributors.
+# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,28 +12,26 @@
 ##############################################################################
 """ Basic Site content type registry
 
-$Id: ContentTypeRegistry.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: ContentTypeRegistry.py 94487 2009-01-03 16:27:40Z yuppie $
 """
 
-import re, os, urllib
+import os
+import re
+import urllib
 
-from AccessControl import ClassSecurityInfo
-from Globals import DTMLFile
-from Globals import InitializeClass
-from Globals import PersistentMapping
+from AccessControl.SecurityInfo import ClassSecurityInfo
+from App.class_init import InitializeClass
+from App.special_dtml import DTMLFile
 from OFS.SimpleItem import SimpleItem
+from Persistence import PersistentMapping
 from zope.interface import implements
 from ZPublisher.mapply import mapply
 
-from interfaces import IContentTypeRegistry
-from interfaces import IContentTypeRegistryPredicate
-from interfaces.ContentTypeRegistry \
-        import ContentTypeRegistry as z2IContentTypeRegistry
-from interfaces.ContentTypeRegistry \
-        import ContentTypeRegistryPredicate as z2IContentTypeRegistryPredicate
-from permissions import ManagePortal
-from utils import _dtmldir
-from utils import getToolByName
+from Products.CMFCore.interfaces import IContentTypeRegistry
+from Products.CMFCore.interfaces import IContentTypeRegistryPredicate
+from Products.CMFCore.permissions import ManagePortal
+from Products.CMFCore.utils import _dtmldir
+from Products.CMFCore.utils import getToolByName
 
 
 class MajorMinorPredicate( SimpleItem ):
@@ -44,7 +42,6 @@ class MajorMinorPredicate( SimpleItem ):
     """
 
     implements(IContentTypeRegistryPredicate)
-    __implements__ = z2IContentTypeRegistryPredicate
 
     major = minor = None
     PREDICATE_TYPE  = 'major_minor'
@@ -133,7 +130,6 @@ class ExtensionPredicate( SimpleItem ):
     """
 
     implements(IContentTypeRegistryPredicate)
-    __implements__ = z2IContentTypeRegistryPredicate
 
     extensions = None
     PREDICATE_TYPE  = 'extension'
@@ -200,7 +196,6 @@ class MimeTypeRegexPredicate( SimpleItem ):
     """
 
     implements(IContentTypeRegistryPredicate)
-    __implements__ = z2IContentTypeRegistryPredicate
 
     pattern         = None
     PREDICATE_TYPE  = 'mimetype_regex'
@@ -259,7 +254,6 @@ class NameRegexPredicate( SimpleItem ):
     """
 
     implements(IContentTypeRegistryPredicate)
-    __implements__ = z2IContentTypeRegistryPredicate
 
     pattern         = None
     PREDICATE_TYPE  = 'name_regex'
@@ -335,7 +329,6 @@ class ContentTypeRegistry( SimpleItem ):
     """
 
     implements(IContentTypeRegistry)
-    __implements__ = z2IContentTypeRegistry
 
     meta_type = 'Content Type Registry'
     id = 'content_type_registry'
