@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Foundation and Contributors.
+# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,45 +12,49 @@
 ##############################################################################
 """ Type registration tool.
 
-$Id: TypesTool.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: TypesTool.py 106204 2009-12-04 10:10:11Z yuppie $
 """
 
 import logging
 from warnings import warn
 
-import Products
-from AccessControl import ClassSecurityInfo
-from AccessControl import getSecurityManager
+from AccessControl.SecurityInfo import ClassSecurityInfo
+from AccessControl.SecurityManagement import getSecurityManager
 from Acquisition import aq_base
 from Acquisition import aq_get
-from Globals import DTMLFile
-from Globals import InitializeClass
-from OFS.Folder import Folder
+from App.class_init import InitializeClass
+from App.special_dtml import DTMLFile
+from OFS.OrderedFolder import OrderedFolder
 from OFS.ObjectManager import IFAwareObjectManager
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
 from zope.component import getUtility
 from zope.component import queryUtility
 from zope.component.interfaces import IFactory
+from zope.container.contained import ObjectAddedEvent
+from zope.container.contained import notifyContainerModified
+from zope.event import notify
 from zope.i18nmessageid import Message
 from zope.interface import implements
+from zope.lifecycleevent import ObjectCreatedEvent
 
-from ActionProviderBase import ActionProviderBase
-from exceptions import AccessControl_Unauthorized
-from exceptions import BadRequest
-from exceptions import zExceptions_Unauthorized
-from interfaces import ITypeInformation
-from interfaces import ITypesTool
-from interfaces.portal_types \
-        import ContentTypeInformation as z2ITypeInformation
-from interfaces.portal_types import portal_types as z2ITypesTool
-from permissions import AccessContentsInformation
-from permissions import ManagePortal
-from permissions import View
-from utils import _checkPermission
-from utils import _dtmldir
-from utils import _wwwdir
-from utils import SimpleItemWithProperties
-from utils import UniqueObject
+from Products.CMFCore.ActionProviderBase import ActionProviderBase
+from Products.CMFCore.exceptions import AccessControl_Unauthorized
+from Products.CMFCore.exceptions import BadRequest
+from Products.CMFCore.exceptions import zExceptions_Unauthorized
+from Products.CMFCore.Expression import Expression
+from Products.CMFCore.interfaces import IAction
+from Products.CMFCore.interfaces import ITypeInformation
+from Products.CMFCore.interfaces import ITypesTool
+from Products.CMFCore.permissions import AccessContentsInformation
+from Products.CMFCore.permissions import AddPortalContent
+from Products.CMFCore.permissions import ManagePortal
+from Products.CMFCore.permissions import View
+from Products.CMFCore.utils import _checkPermission
+from Products.CMFCore.utils import _dtmldir
+from Products.CMFCore.utils import _wwwdir
+from Products.CMFCore.utils import getToolByName
+from Products.CMFCore.utils import SimpleItemWithProperties
+from Products.CMFCore.utils import UniqueObject
 
 logger = logging.getLogger('CMFCore.TypesTool')
 
@@ -62,6 +66,8 @@ class TypeInformation(SimpleItemWithProperties, ActionProviderBase):
     """ Base class for information about a content type.
     """
 
+    implements(IAction)
+
     manage_options = ( SimpleItemWithProperties.manage_options[:1]
                      + ( {'label':'Aliases',
                           'action':'manage_aliases'}, )
@@ -82,13 +88,17 @@ class TypeInformation(SimpleItemWithProperties, ActionProviderBase):
          'label':'Description'},
         {'id':'i18n_domain', 'type': 'string', 'mode':'w',
          'label':'I18n Domain'},
-        {'id':'content_icon', 'type': 'string', 'mode':'w',
-         'label':'Icon'},
+        {'id': 'icon_expr', 'type': 'string', 'mode': 'w',
+         'label': 'Icon (Expression)'},
         {'id':'content_meta_type', 'type': 'string', 'mode':'w',
          'label':'Product meta type'},
         )
 
     _advanced_properties = (
+        {'id': 'add_view_expr', 'type': 'string', 'mode': 'w',
+         'label': 'Add view URL (Expression)'},
+        {'id': 'link_target', 'type': 'string', 'mode': 'w',
+         'label': 'Add view link target'},
         {'id':'immediate_view', 'type': 'string', 'mode':'w',
          'label':'Initial view name'},
         {'id':'global_allow', 'type': 'boolean', 'mode':'w',
@@ -110,12 +120,14 @@ class TypeInformation(SimpleItemWithProperties, ActionProviderBase):
     description = ''
     i18n_domain = ''
     content_meta_type = ''
-    content_icon = ''
+    icon_expr = ''
+    add_view_expr = ''
     immediate_view = ''
     filter_content_types = True
     allowed_content_types = ()
     allow_discussion = False
     global_allow = True
+    link_target = ''
 
     def __init__(self, id, **kw):
 
@@ -132,9 +144,21 @@ class TypeInformation(SimpleItemWithProperties, ActionProviderBase):
             and kw.has_key('meta_type')):
             kw['content_meta_type'] = kw['meta_type']
 
-        if (not kw.has_key('content_icon')
-            and kw.has_key('icon')):
-            kw['content_icon'] = kw['icon']
+        if 'content_icon' in kw or 'icon' in kw:
+            if 'icon' in kw:
+                kw['content_icon'] = kw['icon']
+                warn('TypeInformation got a deprecated argument icon.'
+                     'Support for the icon argument will be removed in '
+                     'CMF 2.4. Please use the icon_expr argument instead.',
+                     DeprecationWarning, stacklevel=2)
+            else:
+                warn('TypeInformation got a deprecated argument content_icon.'
+                     'Support for the content_icon argument will be removed in '
+                     'CMF 2.4. Please use the icon_expr argument instead.',
+                     DeprecationWarning, stacklevel=2)
+
+            if 'icon_expr' not in kw:
+                kw['icon_expr'] = 'string:${portal_url}/%s' % kw['content_icon']
 
         self.manage_changeProperties(**kw)
 
@@ -148,6 +172,8 @@ class TypeInformation(SimpleItemWithProperties, ActionProviderBase):
                 , permission=action.get( 'permissions', () )
                 , category=action.get('category', 'object')
                 , visible=action.get('visible', True)
+                , icon_expr=action.get('icon_expr', '')
+                , link_target=action.get('link_target', '')
                 )
 
         self.setMethodAliases(kw.get('aliases', {}))
@@ -214,10 +240,21 @@ class TypeInformation(SimpleItemWithProperties, ActionProviderBase):
 
     security.declareProtected(View, 'getIcon')
     def getIcon(self):
+        """ Returns the icon for this content object.
         """
-            Returns the icon for this content object.
+        warn('getIcon() is deprecated and provides only limited backwards '
+             'compatibility. It will be removed in CMF 2.4. Please use '
+             'getIconExprObject() instead.',
+             DeprecationWarning, stacklevel=2)
+        if self.icon_expr.startswith('string:${portal_url}/'):
+            return self.icon_expr[len('string:${portal_url}/'):]
+        return self.icon_expr
+
+    security.declarePrivate('getIconExprObject')
+    def getIconExprObject(self):
+        """ Get the expression object representing the icon for this type.
         """
-        return self.content_icon
+        return getattr(self, 'icon_expr_object', None)
 
     security.declarePublic('allowType')
     def allowType( self, contentType ):
@@ -273,24 +310,7 @@ class TypeInformation(SimpleItemWithProperties, ActionProviderBase):
         if not self.isConstructionAllowed(container):
             raise AccessControl_Unauthorized('Cannot create %s' % self.getId())
 
-        ob = self._constructInstance(container, id, *args, **kw)
-
-        return self._finishConstruction(ob)
-
-    security.declarePrivate('_finishConstruction')
-    def _finishConstruction(self, ob):
-        """
-            Finish the construction of a content object.
-            Set its portal_type, insert it into the workflows.
-        """
-        if hasattr(ob, '_setPortalTypeName'):
-            ob._setPortalTypeName(self.getId())
-
-        if hasattr(aq_base(ob), 'notifyWorkflowCreated'):
-            ob.notifyWorkflowCreated()
-
-        ob.reindexObject()
-        return ob
+        return self._constructInstance(container, id, *args, **kw)
 
     security.declareProtected(ManagePortal, 'getMethodAliases')
     def getMethodAliases(self):
@@ -329,6 +349,80 @@ class TypeInformation(SimpleItemWithProperties, ActionProviderBase):
         if isinstance(method_id, tuple):
             method_id = method_id[0]
         return method_id
+    
+    security.declarePrivate('_checkWorkflowAllowed')
+    def _checkWorkflowAllowed(self, container):
+        """ Check if a workflow veto object creation
+        """
+        wtool = getToolByName(self, 'portal_workflow', None)
+        if wtool is None:
+            return True
+        
+        type_id = self.getId()
+        workflows = wtool.getWorkflowsFor(type_id)
+        for workflow in workflows:
+            # DCWorkflow workflows define an instance creation guard
+            guard = getattr(workflow, 'allowCreate', None)
+
+            if guard is None:
+                continue
+
+            if not guard(container, type_id):
+                return False
+        
+        return True        
+
+    #
+    #   'IAction' interface methods
+    #
+    security.declarePrivate('getInfoData')
+    def getInfoData(self):
+        """ Get the data needed to create an ActionInfo.
+        """
+        lazy_keys = ['available', 'allowed']
+        lazy_map = {}
+
+        lazy_map['id'] = self.getId()
+        lazy_map['category'] = 'folder/add'
+        lazy_map['title'] = self.Title()
+        lazy_map['description'] = self.Description()
+        if self.add_view_expr:
+            lazy_map['url'] = self.add_view_expr_object
+            lazy_keys.append('url')
+        else:
+            lazy_map['url'] = ''
+        if self.icon_expr:
+            lazy_map['icon'] = self.icon_expr_object
+            lazy_keys.append('icon')
+        else:
+            lazy_map['icon'] = ''
+        lazy_map['link_target'] = self.link_target
+        lazy_map['visible'] = True
+        lazy_map['available'] = self._checkAvailable
+        lazy_map['allowed'] = self._checkAllowed
+
+        return (lazy_map, lazy_keys)
+
+    def _setPropValue(self, id, value):
+        self._wrapperCheck(value)
+        if isinstance(value, list):
+            value = tuple(value)
+        setattr(self, id, value)
+        if value and id.endswith('_expr'):
+            setattr(self, '%s_object' % id, Expression(value))
+
+    def _checkAvailable(self, ec):
+        """ Check if the action is available in the current context.
+        """
+        return ec.contexts['folder'].getTypeInfo().allowType(self.getId())
+
+    def _checkAllowed(self, ec):
+        """ Check if the action is allowed in the current context.
+        """
+        container = ec.contexts['folder']
+        if not _checkPermission(AddPortalContent, container):
+            return False
+        return self.isConstructionAllowed(container)
 
 InitializeClass(TypeInformation)
 
@@ -339,7 +433,6 @@ class FactoryTypeInformation(TypeInformation):
     """
 
     implements(ITypeInformation)
-    __implements__ = z2ITypeInformation
 
     security = ClassSecurityInfo()
 
@@ -410,11 +503,15 @@ class FactoryTypeInformation(TypeInformation):
 
         c. Does the current user have the permission required in
         order to invoke the factory method?
+        
+        d. Do all workflows authorize the creation?
         """
+        ti_check = False
+
         if self.product:
             # oldstyle factory
             m = self._queryFactoryMethod(container)
-            return (m is not None)
+            ti_check = m is not None
 
         elif container is not None:
             # newstyle factory
@@ -423,17 +520,20 @@ class FactoryTypeInformation(TypeInformation):
                 for d in container.all_meta_types():
                     if d['name'] == self.content_meta_type:
                         sm = getSecurityManager()
-                        return sm.checkPermission(d['permission'], container)
+                        ti_check = sm.checkPermission(d['permission'],
+                                                      container)
+                        break
 
-        return False
+        if not ti_check:
+            return False
+        else :
+            return self._checkWorkflowAllowed(container)
 
     security.declarePrivate('_constructInstance')
     def _constructInstance(self, container, id, *args, **kw):
         """Build a bare instance of the appropriate type.
 
         Does not do any security checks.
-
-        Returns the object without calling _finishConstruction().
         """
         # XXX: this method violates the rules for tools/utilities:
         # it depends on self.REQUEST
@@ -450,15 +550,25 @@ class FactoryTypeInformation(TypeInformation):
                 newid = m(id, *args, **kw)
             # allow factory to munge ID
             newid = newid or id
+            obj = container._getOb(newid)
+            if hasattr(obj, '_setPortalTypeName'):
+                obj._setPortalTypeName(self.getId())
+            notify(ObjectCreatedEvent(obj))
+            notify(ObjectAddedEvent(obj, container, newid))
+            notifyContainerModified(container)
 
         else:
             # newstyle factory
             factory = getUtility(IFactory, self.factory)
             obj = factory(id, *args, **kw)
+            if hasattr(obj, '_setPortalTypeName'):
+                obj._setPortalTypeName(self.getId())
+            notify(ObjectCreatedEvent(obj))
             rval = container._setObject(id, obj)
             newid = isinstance(rval, basestring) and rval or id
+            obj = container._getOb(newid)
 
-        return container._getOb(newid)
+        return obj
 
 InitializeClass(FactoryTypeInformation)
 
@@ -469,7 +579,6 @@ class ScriptableTypeInformation(TypeInformation):
     """
 
     implements(ITypeInformation)
-    __implements__ = z2ITypeInformation
 
     security = ClassSecurityInfo()
 
@@ -495,15 +604,13 @@ class ScriptableTypeInformation(TypeInformation):
         permission = self.permission
         if permission and not _checkPermission( permission, container ):
             return 0
-        return 1
+        return self._checkWorkflowAllowed(container)
 
     security.declarePrivate('_constructInstance')
     def _constructInstance(self, container, id, *args, **kw):
         """Build a bare instance of the appropriate type.
 
         Does not do any security checks.
-
-        Returns the object without calling _finishConstruction().
         """
         constructor = self.restrictedTraverse( self.constructor_path )
 
@@ -514,7 +621,13 @@ class ScriptableTypeInformation(TypeInformation):
         constructor = aq_base(constructor).__of__( container )
 
         id = str(id)
-        return constructor(container, id, *args, **kw)
+        obj = constructor(container, id, *args, **kw)
+        if hasattr(obj, '_setPortalTypeName'):
+            obj._setPortalTypeName(self.getId())
+        notify(ObjectAddedEvent(obj, container, obj.getId()))
+        notifyContainerModified(container)
+        return obj
+
 
 InitializeClass(ScriptableTypeInformation)
 
@@ -527,14 +640,13 @@ allowedTypes = [
     ]
 
 
-class TypesTool(UniqueObject, IFAwareObjectManager, Folder,
+class TypesTool(UniqueObject, IFAwareObjectManager, OrderedFolder,
                 ActionProviderBase):
 
     """ Provides a configurable registry of portal content types.
     """
 
     implements(ITypesTool)
-    __implements__ = (z2ITypesTool, ActionProviderBase.__implements__)
 
     id = 'portal_types'
     meta_type = 'CMF Types Tool'
@@ -542,13 +654,13 @@ class TypesTool(UniqueObject, IFAwareObjectManager, Folder,
 
     security = ClassSecurityInfo()
 
-    manage_options = ( Folder.manage_options[:1]
+    manage_options = ( OrderedFolder.manage_options[:1]
                      + ( {'label':'Aliases',
                           'action':'manage_aliases'}, )
                      + ActionProviderBase.manage_options
                      + ( {'label':'Overview',
                           'action':'manage_overview'}, )
-                     + Folder.manage_options[1:]
+                     + OrderedFolder.manage_options[1:]
                      )
 
     #
@@ -566,6 +678,7 @@ class TypesTool(UniqueObject, IFAwareObjectManager, Folder,
     def all_meta_types(self, interfaces=None):
         # this is a workaround and should be removed again if allowedTypes
         # have an interface we can use in _product_interfaces
+        import Products
         all = TypesTool.inheritedAttribute('all_meta_types')(self)
         others = [ mt for mt in Products.meta_types
                    if mt['name'] in allowedTypes ]
@@ -580,6 +693,7 @@ class TypesTool(UniqueObject, IFAwareObjectManager, Folder,
         """Create a TypeInformation in self.
         """
         # BBB: typeinfo_name is ignored
+        import Products
         if not id:
             raise BadRequest('An id is required.')
         for mt in Products.meta_types:
@@ -658,9 +772,10 @@ class TypesTool(UniqueObject, IFAwareObjectManager, Folder,
                 rval.append(t)
             elif getattr(aq_base(t), '_isTypeInformation', 0):
                 # BBB
-                warn("The '_isTypeInformation' marker attribute is deprecated, "
-                     "and will be removed in CMF 2.3.  Please mark the "
-                     "instance with the 'ITypeInformation' interface instead.",
+                warn("The '_isTypeInformation' marker attribute is "
+                     "deprecated, and will be removed in CMF 2.3.  "
+                     "Please mark the instance with the 'ITypeInformation' "
+                     "interface instead.",
                      DeprecationWarning, stacklevel=2)
                 rval.append(t)
         # Skip items with no ID:  old signal for "not ready"
@@ -726,7 +841,12 @@ class TypesTool(UniqueObject, IFAwareObjectManager, Folder,
     def listActions(self, info=None, object=None):
         """ List all the actions defined by a provider.
         """
-        actions = list( self._actions )
+        oldstyle_actions = self._actions or ()
+        if oldstyle_actions:
+            warn('Old-style actions are deprecated and will be removed in CMF '
+                 '2.4. Use Action and Action Category objects instead.',
+                 DeprecationWarning, stacklevel=2)
+        actions = list(oldstyle_actions)
 
         if object is None and info is not None:
             object = info.object
@@ -735,6 +855,10 @@ class TypesTool(UniqueObject, IFAwareObjectManager, Folder,
             if type_info is not None:
                 actions.extend( type_info.listActions(info, object) )
 
+        add_actions = [ ti for ti in self.objectValues()
+                        if IAction.providedBy(ti) ]
+        actions.extend(add_actions)
+
         return actions
 
     security.declareProtected(ManagePortal, 'listMethodAliasKeys')
