@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Foundation and Contributors.
+# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,7 +12,7 @@
 ##############################################################################
 """Catalog tool setup handlers.
 
-$Id: catalog.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: catalog.py 107359 2009-12-30 15:33:55Z jens $
 """
 
 from Products.GenericSetup.utils import exportObjects
@@ -25,7 +25,11 @@ def importCatalogTool(context):
     """Import catalog tool.
     """
     site = context.getSite()
-    tool = getToolByName(site, 'portal_catalog')
+    tool = getToolByName(site, 'portal_catalog', None)
+    if tool is None:
+        logger = context.getLogger('catalog')
+        logger.debug('Nothing to import.')
+        return
 
     importObjects(tool, '', context)
 
@@ -36,7 +40,7 @@ def exportCatalogTool(context):
     tool = getToolByName(site, 'portal_catalog', None)
     if tool is None:
         logger = context.getLogger('catalog')
-        logger.info('Nothing to export.')
+        logger.debug('Nothing to export.')
         return
 
     exportObjects(tool, '', context)
