@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Foundation and Contributors.
+# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,7 +12,7 @@
 ##############################################################################
 """Mailhost setup handlers.
 
-$Id: mailhost.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: mailhost.py 107359 2009-12-30 15:33:55Z jens $
 """
 
 from Products.MailHost.interfaces import IMailHost
@@ -27,7 +27,11 @@ def importMailHost(context):
     """Import mailhost settings from an XML file.
     """
     sm = getSiteManager(context.getSite())
-    tool = sm.getUtility(IMailHost)
+    tool = sm.queryUtility(IMailHost)
+    if tool is None:
+        logger = context.getLogger('mailhost')
+        logger.debug('Nothing to import.')
+        return
 
     importObjects(tool, '', context)
 
@@ -38,7 +42,7 @@ def exportMailHost(context):
     tool = sm.queryUtility(IMailHost)
     if tool is None:
         logger = context.getLogger('mailhost')
-        logger.info('Nothing to export.')
+        logger.debug('Nothing to export.')
         return
 
     exportObjects(tool, '', context)
