@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Foundation and Contributors.
+# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,7 +12,7 @@
 ##############################################################################
 """Actions tool node adapter unit tests.
 
-$Id: test_actions.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: test_actions.py 105897 2009-11-20 07:57:56Z yuppie $
 """
 
 import unittest
@@ -22,12 +22,10 @@ from Acquisition import aq_parent
 from Acquisition import Implicit
 from OFS.OrderedFolder import OrderedFolder
 from zope.interface import implements
-from zope.interface import Interface
 
 from Products.CMFCore.ActionProviderBase import ActionProviderBase
+from Products.CMFCore.interfaces import IActionProvider
 from Products.CMFCore.interfaces import IActionsTool
-from Products.CMFCore.interfaces.portal_actions \
-    import ActionProvider as IActionProvider
 from Products.CMFCore.testing import ExportImportZCMLLayer
 from Products.CMFCore.tests.base.dummy import DummySite
 from Products.GenericSetup.testing import BodyAdapterTestCase
@@ -41,6 +39,7 @@ _ACTION_XML = """\
  <property name="title">Foo</property>
  <property name="description"></property>
  <property name="url_expr">string:${object_url}/foo</property>
+ <property name="link_target"></property>
  <property name="icon_expr"></property>
  <property name="available_expr">python:1</property>
  <property name="permissions"/>
@@ -55,6 +54,7 @@ _ACTIONCATEGORY_XML = """\
   <property name="title"></property>
   <property name="description"></property>
   <property name="url_expr"></property>
+  <property name="link_target"></property>
   <property name="icon_expr"></property>
   <property name="available_expr"></property>
   <property name="permissions"/>
@@ -74,6 +74,7 @@ _ACTIONSTOOL_BODY = """\
    <property name="title" i18n:translate=""></property>
    <property name="description" i18n:translate=""></property>
    <property name="url_expr"></property>
+   <property name="link_target"></property>
    <property name="icon_expr"></property>
    <property name="available_expr"></property>
    <property name="permissions"/>
@@ -91,11 +92,18 @@ _EMPTY_EXPORT = """\
 </object>
 """
 
-_NORMAL_EXPORT = """\
+_OLD_EXPORT = """\
 <?xml version="1.0"?>
 <object name="portal_actions" meta_type="CMF Actions Tool"
    xmlns:i18n="http://xml.zope.org/namespaces/i18n">
- <action-provider name="portal_actions"/>
+ <action-provider name="portal_actions">
+  <action action_id="baz"
+          title="Baz"
+          url_expr="string:${object_url}/baz"
+          condition_expr="python:1"
+          category="dummy"
+          visible="True"/>
+ </action-provider>
  <action-provider name="portal_foo">
   <action action_id="foo"
           title="Foo"
@@ -117,33 +125,42 @@ _NORMAL_EXPORT = """\
 </object>
 """
 
+_NORMAL_EXPORT = """\
+<?xml version="1.0"?>
+<object name="portal_actions" meta_type="CMF Actions Tool"
+   xmlns:i18n="http://xml.zope.org/namespaces/i18n">
+ <action-provider name="portal_actions">
+  <action action_id="baz"
+          title="Baz"
+          url_expr="string:${object_url}/baz"
+          condition_expr="python:1"
+          category="dummy"
+          visible="True"/>
+ </action-provider>
+ <action-provider name="portal_foo"/>
+ <action-provider name="portal_bar"/>
+</object>
+"""
+
 _NEWSYTLE_EXPORT = """\
 <?xml version="1.0"?>
 <object name="portal_actions" meta_type="CMF Actions Tool"
    xmlns:i18n="http://xml.zope.org/namespaces/i18n">
  <action-provider name="portal_actions"/>
+ <action-provider name="portal_foo"/>
+ <action-provider name="portal_bar"/>
  <object name="dummy" meta_type="CMF Action Category">
   <property name="title"></property>
-  <object name="foo" meta_type="CMF Action">
-   <property name="title">Foo</property>
+  <object name="baz" meta_type="CMF Action">
+   <property name="title">Baz</property>
    <property name="description"></property>
-   <property name="url_expr">string:${object_url}/foo</property>
+   <property name="url_expr">string:${object_url}/baz</property>
+   <property name="link_target"></property>
    <property name="icon_expr"></property>
    <property name="available_expr">python:1</property>
    <property name="permissions"></property>
    <property name="visible">True</property>
   </object>
-  <object name="bar" meta_type="CMF Action">
-   <property name="title">Bar</property>
-   <property name="description"></property>
-   <property name="url_expr">string:${object_url}/bar</property>
-   <property name="icon_expr"></property>
-   <property name="available_expr">python:0</property>
-   <property name="permissions">
-    <element value="Manage portal"/>
-   </property>
-   <property name="visible">False</property>
-  </object>
  </object>
 </object>
 """
@@ -159,6 +176,7 @@ _I18N_IMPORT = """\
    <property name="title" i18n:translate="">Foo</property>
    <property name="description" i18n:translate=""></property>
    <property name="url_expr">string:${object_url}/foo</property>
+   <property name="link_target"></property>
    <property name="icon_expr"></property>
    <property name="available_expr">python:1</property>
    <property name="permissions"></property>
@@ -182,8 +200,8 @@ _INSERT_IMPORT = """\
    <element value="View" /></property>
   <property name="visible">True</property>
  </object>
- <object name="foo" insert-after="*">
-  <property name="icon_expr">string:foo_icon.png</property>
+ <object name="baz" insert-after="*">
+  <property name="icon_expr">string:baz_icon.png</property>
  </object>
  </object>
 </object>
@@ -201,7 +219,7 @@ _REMOVE_IMPORT = """\
 
 class DummyTool(OrderedFolder, ActionProviderBase):
 
-    __implements__ = IActionProvider
+    implements(IActionProvider)
 
 
 class DummyUser(Implicit):
@@ -238,7 +256,7 @@ class DummyActionsTool(DummyTool):
         self._providers = [ x for x in self._providers if x != provider_name ]
 
 
-class ActionNodeAdapterTests(NodeAdapterTestCase):
+class ActionNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):
 
     layer = ExportImportZCMLLayer
 
@@ -271,12 +289,11 @@ class ActionNodeAdapterTests(NodeAdapterTestCase):
     def setUp(self):
         from Products.CMFCore.ActionInformation import Action
 
-        NodeAdapterTestCase.setUp(self)
         self._obj = Action('foo_action')
         self._XML = _ACTION_XML
 
 
-class ActionCategoryNodeAdapterTests(NodeAdapterTestCase):
+class ActionCategoryNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):
 
     layer = ExportImportZCMLLayer
 
@@ -298,12 +315,11 @@ class ActionCategoryNodeAdapterTests(NodeAdapterTestCase):
     def setUp(self):
         from Products.CMFCore.ActionInformation import ActionCategory
 
-        NodeAdapterTestCase.setUp(self)
         self._obj = ActionCategory('foo_category')
         self._XML = _ACTIONCATEGORY_XML
 
 
-class ActionsToolXMLAdapterTests(BodyAdapterTestCase):
+class ActionsToolXMLAdapterTests(BodyAdapterTestCase, unittest.TestCase):
 
     layer = ExportImportZCMLLayer
 
@@ -331,7 +347,6 @@ class ActionsToolXMLAdapterTests(BodyAdapterTestCase):
     def setUp(self):
         from Products.CMFCore.ActionsTool import ActionsTool
 
-        BodyAdapterTestCase.setUp(self)
         site = DummySite('site')
         site._setObject('portal_actions', ActionsTool('portal_actions'))
         self._obj = site.portal_actions
@@ -400,6 +415,15 @@ class exportActionProvidersTests(_ActionSetup):
                 import exportActionProviders
 
         site = self._initSite()
+        # Set up an old action for added difficulty
+        site.portal_actions.addAction(id='baz',
+                                      name='Baz',
+                                      action='baz',
+                                      condition='python:1',
+                                      permission=(),
+                                      category='dummy',
+                                      visible=1)
+
         context = DummyExportContext(site)
         exportActionProviders(context)
 
@@ -492,18 +516,21 @@ class importActionProvidersTests(_ActionSetup):
         self.failUnless('portal_actions' in atool.listActionProviders())
 
         context = DummyImportContext(site)
-        context._files['actions.xml'] = _NORMAL_EXPORT
+        context._files['actions.xml'] = _OLD_EXPORT
         importActionProviders(context)
 
-        self.assertEqual(len(atool.listActionProviders()), 1)
-        self.failIf('portal_foo' in atool.listActionProviders())
+        self.assertEqual(len(atool.listActionProviders()), 3)
+        self.failUnless('portal_bar' in atool.listActionProviders())
+        self.failUnless('portal_foo' in atool.listActionProviders())
         self.failUnless('portal_actions' in atool.listActionProviders())
 
         self.assertEqual(len(atool.objectIds()), 1)
         self.failUnless('dummy' in atool.objectIds())
-        self.assertEqual(len(atool.dummy.objectIds()) , 2)
-        self.failUnless('foo' in atool.dummy.objectIds())
-        self.failUnless('bar' in atool.dummy.objectIds())
+        # Only one action appears. The importer only deals with actions
+        # defined by the actions tool. Other tools are responsible for
+        # exporting/importing actions themselves.
+        self.assertEqual(len(atool.dummy.objectIds()) , 1)
+        self.failUnless('baz' in atool.dummy.objectIds())
         self.failIf(foo.listActions())
         self.failIf(bar.listActions())
 
@@ -556,19 +583,19 @@ class importActionProvidersTests(_ActionSetup):
         context._files['actions.xml'] = _NEWSYTLE_EXPORT
         importActionProviders(context)
 
-        self.assertEqual(len(atool.listActionProviders()), 1)
+        self.assertEqual(len(atool.listActionProviders()), 3)
         self.assertEqual(atool.objectIds(), ['dummy'])
-        self.assertEqual(atool.dummy.objectIds(), ['foo', 'bar'])
-        self.assertEqual(atool.dummy.foo.icon_expr, '')
+        self.assertEqual(atool.dummy.objectIds(), ['baz'])
+        self.assertEqual(atool.dummy.baz.icon_expr, '')
 
         context = DummyImportContext(site, False)
         context._files['actions.xml'] = _INSERT_IMPORT
         importActionProviders(context)
 
-        self.assertEqual(len(atool.listActionProviders()), 1)
+        self.assertEqual(len(atool.listActionProviders()), 3)
         self.assertEqual(atool.objectIds(), ['dummy'])
-        self.assertEqual(atool.dummy.objectIds(), ['spam', 'bar', 'foo'])
-        self.assertEqual(atool.dummy.foo.icon_expr, 'string:foo_icon.png')
+        self.assertEqual(atool.dummy.objectIds(), ['spam', 'baz'])
+        self.assertEqual(atool.dummy.baz.icon_expr, 'string:baz_icon.png')
 
     def test_remove_skip_purge(self):
         from Products.CMFCore.exportimport.actions \
