@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Foundation and Contributors.
+# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,7 +12,7 @@
 ##############################################################################
 """Types tool xml adapter and setup handler unit tests.
 
-$Id: test_typeinfo.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: test_typeinfo.py 106204 2009-12-04 10:10:11Z yuppie $
 """
 
 import unittest
@@ -29,6 +29,7 @@ from Products.CMFCore.permissions import View
 from Products.CMFCore.permissions import AccessContentsInformation
 from Products.CMFCore.permissions import ModifyPortalContent
 from Products.CMFCore.testing import ExportImportZCMLLayer
+from Products.CMFCore.tests.base.testcase import WarningInterceptor
 from Products.CMFCore.TypesTool import FactoryTypeInformation
 from Products.CMFCore.TypesTool import ScriptableTypeInformation
 from Products.CMFCore.TypesTool import TypesTool
@@ -39,10 +40,12 @@ _FTI_BODY = """\
    xmlns:i18n="http://xml.zope.org/namespaces/i18n">
  <property name="title"></property>
  <property name="description"></property>
- <property name="content_icon"></property>
+ <property name="icon_expr"></property>
  <property name="content_meta_type"></property>
  <property name="product"></property>
  <property name="factory"></property>
+ <property name="add_view_expr"></property>
+ <property name="link_target"></property>
  <property name="immediate_view"></property>
  <property name="global_allow">True</property>
  <property name="filter_content_types">True</property>
@@ -51,8 +54,8 @@ _FTI_BODY = """\
  <alias from="(Default)" to="foo"/>
  <alias from="view" to="foo"/>
  <action title="Foo" action_id="foo_action" category="Bar"
-    condition_expr="python:1" url_expr="string:${object_url}/foo"
-    visible="True"/>
+    condition_expr="python:1" icon_expr="string:${portal_url}/icon.png"
+    link_target="" url_expr="string:${object_url}/foo" visible="True"/>
 </object>
 """
 
@@ -70,9 +73,11 @@ _TI_LIST = ({
     'description':           'Foo things',
     'i18n_domain':           'foo_domain',
     'content_meta_type':     'Foo Thing',
-    'content_icon':          'foo.png',
+    'icon_expr':             'string:${portal_url}/foo.png',
     'product':               'CMFSetup',
     'factory':               'addFoo',
+    'add_view_expr':         'string:${folder_url}/foo_add_view',
+    'link_target':           '_new',
     'immediate_view':        'foo_view',
     'filter_content_types':  False,
     'allowed_content_types': (),
@@ -84,16 +89,20 @@ _TI_LIST = ({
     'actions': ({'id':     'view',
                  'title':  'View',
                  'action': 'string:${object_url}/foo_view',
+                 'icon_expr': 'string:${portal_url}/preview_icon.png',
+                 'link_target': '_new',
                  'permissions': (View,),
                  },
                 {'id':     'edit',
                  'title':  'Edit',
                  'action': 'string:${object_url}/foo_edit_form',
+                 'icon_expr': 'string:${portal_url}/edit_icon.png',
                  'permissions': (ModifyPortalContent,),
                  },
                 {'id':     'metadata',
                  'title':  'Metadata',
                  'action': 'string:${object_url}/metadata_edit_form',
+                 'icon_expr': 'string:${portal_url}/metadata_icon.png',
                  'permissions': (ModifyPortalContent,),
                  },
                 ),
@@ -102,9 +111,11 @@ _TI_LIST = ({
     'title':                 'Bar',
     'description':           'Bar things',
     'content_meta_type':     'Bar Thing',
-    'content_icon':          'bar.png',
+    'icon_expr':             'string:${portal_url}/bar.png',
     'constructor_path':      'make_bar',
     'permission':            'Add portal content',
+    'add_view_expr':         'string:${folder_url}/bar_add_view',
+    'link_target':           '',
     'immediate_view':        'bar_view',
     'filter_content_types':  True,
     'allowed_content_types': ('foo',),
@@ -154,8 +165,8 @@ _NORMAL_TOOL_EXPORT = """\
 <?xml version="1.0"?>
 <object name="portal_types" meta_type="CMF Types Tool">
  <property name="title"/>
- <object name="bar" meta_type="Scriptable Type Information"/>
  <object name="foo" meta_type="Factory-based Type Information"/>
+ <object name="bar" meta_type="Scriptable Type Information"/>
 </object>
 """
 
@@ -163,8 +174,8 @@ _FILENAME_EXPORT = """\
 <?xml version="1.0"?>
 <object name="portal_types" meta_type="CMF Types Tool">
  <property name="title"/>
- <object name="bar object" meta_type="Scriptable Type Information"/>
  <object name="foo object" meta_type="Factory-based Type Information"/>
+ <object name="bar object" meta_type="Scriptable Type Information"/>
 </object>
 """
 
@@ -181,10 +192,12 @@ _FOO_EXPORT = """\
    i18n:domain="foo_domain" xmlns:i18n="http://xml.zope.org/namespaces/i18n">
  <property name="title" i18n:translate="">Foo</property>
  <property name="description" i18n:translate="">Foo things</property>
- <property name="content_icon">foo.png</property>
+ <property name="icon_expr">string:${portal_url}/foo.png</property>
  <property name="content_meta_type">Foo Thing</property>
  <property name="product">CMFSetup</property>
  <property name="factory">addFoo</property>
+ <property name="add_view_expr">string:${folder_url}/foo_add_view</property>
+ <property name="link_target">_new</property>
  <property name="immediate_view">foo_view</property>
  <property name="global_allow">False</property>
  <property name="filter_content_types">False</property>
@@ -193,15 +206,20 @@ _FOO_EXPORT = """\
  <alias from="(Default)" to="foo_view"/>
  <alias from="view" to="foo_view"/>
  <action title="View" action_id="view" category="object" condition_expr=""
-    url_expr="string:${object_url}/foo_view" visible="True">
+    url_expr="string:${object_url}/foo_view" 
+    icon_expr="string:${portal_url}/preview_icon.png" link_target="_new" 
+    visible="True">
   <permission value="View"/>
  </action>
  <action title="Edit" action_id="edit" category="object" condition_expr=""
-    url_expr="string:${object_url}/foo_edit_form" visible="True">
+    url_expr="string:${object_url}/foo_edit_form" 
+    icon_expr="string:${portal_url}/edit_icon.png" link_target="" 
+    visible="True">
   <permission value="Modify portal content"/>
  </action>
  <action title="Metadata" action_id="metadata" category="object"
     condition_expr="" url_expr="string:${object_url}/metadata_edit_form"
+    icon_expr="string:${portal_url}/metadata_icon.png" link_target="" 
     visible="True">
   <permission value="Modify portal content"/>
  </action>
@@ -214,10 +232,12 @@ _BAR_EXPORT = """\
    xmlns:i18n="http://xml.zope.org/namespaces/i18n">
  <property name="title">Bar</property>
  <property name="description">Bar things</property>
- <property name="content_icon">bar.png</property>
+ <property name="icon_expr">string:${portal_url}/bar.png</property>
  <property name="content_meta_type">Bar Thing</property>
  <property name="permission">Add portal content</property>
  <property name="constructor_path">make_bar</property>
+ <property name="add_view_expr">string:${folder_url}/bar_add_view</property>
+ <property name="link_target"/>
  <property name="immediate_view">bar_view</property>
  <property name="global_allow">True</property>
  <property name="filter_content_types">True</property>
@@ -228,21 +248,67 @@ _BAR_EXPORT = """\
  <alias from="(Default)" to="bar_view"/>
  <alias from="view" to="bar_view"/>
  <action title="View" action_id="view" category="object" condition_expr=""
-    url_expr="string:${object_url}/bar_view" visible="True">
+    url_expr="string:${object_url}/bar_view" 
+    icon_expr="" link_target="" visible="True">
   <permission value="View"/>
  </action>
  <action title="Edit" action_id="edit" category="object" condition_expr=""
-    url_expr="string:${object_url}/bar_edit_form" visible="True">
+    url_expr="string:${object_url}/bar_edit_form"
+    icon_expr="" link_target="" visible="True">
   <permission value="Modify portal content"/>
  </action>
  <action title="Contents" action_id="contents" category="object"
     condition_expr="" url_expr="string:${object_url}/folder_contents"
-    visible="True">
+    icon_expr="" link_target="" visible="True">
   <permission value="Access contents information"/>
  </action>
  <action title="Metadata" action_id="metadata" category="object"
     condition_expr="" url_expr="string:${object_url}/metadata_edit_form"
-    visible="True">
+    icon_expr="" link_target="" visible="True">
+  <permission value="Modify portal content"/>
+ </action>
+</object>
+"""
+
+_BAR_CMF21_IMPORT = """\
+<?xml version="1.0"?>
+<object name="%s" meta_type="Scriptable Type Information"
+   xmlns:i18n="http://xml.zope.org/namespaces/i18n">
+ <property name="title">Bar</property>
+ <property name="description">Bar things</property>
+ <property name="content_icon">bar.png</property>
+ <property name="content_meta_type">Bar Thing</property>
+ <property name="permission">Add portal content</property>
+ <property name="constructor_path">make_bar</property>
+ <property name="add_view_expr">string:${folder_url}/bar_add_view</property>
+ <property name="link_target"/>
+ <property name="immediate_view">bar_view</property>
+ <property name="global_allow">True</property>
+ <property name="filter_content_types">True</property>
+ <property name="allowed_content_types">
+  <element value="foo"/>
+ </property>
+ <property name="allow_discussion">True</property>
+ <alias from="(Default)" to="bar_view"/>
+ <alias from="view" to="bar_view"/>
+ <action title="View" action_id="view" category="object" condition_expr=""
+    url_expr="string:${object_url}/bar_view" 
+    icon_expr="" link_target="" visible="True">
+  <permission value="View"/>
+ </action>
+ <action title="Edit" action_id="edit" category="object" condition_expr=""
+    url_expr="string:${object_url}/bar_edit_form"
+    icon_expr="" link_target="" visible="True">
+  <permission value="Modify portal content"/>
+ </action>
+ <action title="Contents" action_id="contents" category="object"
+    condition_expr="" url_expr="string:${object_url}/folder_contents"
+    icon_expr="" link_target="" visible="True">
+  <permission value="Access contents information"/>
+ </action>
+ <action title="Metadata" action_id="metadata" category="object"
+    condition_expr="" url_expr="string:${object_url}/metadata_edit_form"
+    icon_expr="" link_target="" visible="True">
   <permission value="Modify portal content"/>
  </action>
 </object>
@@ -255,7 +321,8 @@ _UPDATE_FOO_IMPORT = """\
 """
 
 
-class TypeInformationXMLAdapterTests(BodyAdapterTestCase):
+class TypeInformationXMLAdapterTests(
+    BodyAdapterTestCase, unittest.TestCase, WarningInterceptor):
 
     layer = ExportImportZCMLLayer
 
@@ -268,7 +335,8 @@ class TypeInformationXMLAdapterTests(BodyAdapterTestCase):
     def _populate(self, obj):
         obj.setMethodAliases({'(Default)': 'foo', 'view': 'foo'})
         obj.addAction('foo_action', 'Foo', 'string:${object_url}/foo',
-                      'python:1', (), 'Bar')
+                      'python:1', (), 'Bar', 
+                      icon_expr="string:${portal_url}/icon.png")
 
     def _verifyImport(self, obj):
         self.assertEqual(type(obj._aliases), dict)
@@ -286,14 +354,21 @@ class TypeInformationXMLAdapterTests(BodyAdapterTestCase):
         self.assertEqual(obj._actions[0].category, 'Bar')
         self.assertEqual(type(obj._actions[0].condition.text), str)
         self.assertEqual(obj._actions[0].condition.text, 'python:1')
+        self.assertEqual(type(obj._actions[0].icon_expr.text), str)
+        self.assertEqual(obj._actions[0].icon_expr.text, 
+                                  "string:${portal_url}/icon.png")
 
     def setUp(self):
-        BodyAdapterTestCase.setUp(self)
         self._obj = FactoryTypeInformation('foo_fti')
         self._BODY = _FTI_BODY
+        self._trap_warning_output()
+
+    def tearDown(self):
+        self._free_warning_output()
 
 
-class TypesToolXMLAdapterTests(BodyAdapterTestCase):
+class TypesToolXMLAdapterTests(
+    BodyAdapterTestCase, unittest.TestCase, WarningInterceptor):
 
     layer = ExportImportZCMLLayer
 
@@ -307,12 +382,22 @@ class TypesToolXMLAdapterTests(BodyAdapterTestCase):
         obj._setObject('foo_type', FactoryTypeInformation('foo_type'))
 
     def setUp(self):
-        BodyAdapterTestCase.setUp(self)
         self._obj = TypesTool()
         self._BODY = _TYPESTOOL_BODY
+        self._trap_warning_output()
+
+    def tearDown(self):
+        self._free_warning_output()
 
 
-class _TypeInfoSetup(BaseRegistryTests):
+class _TypeInfoSetup(BaseRegistryTests, WarningInterceptor):
+
+    def setUp(self):
+        BaseRegistryTests.setUp(self)
+        self._trap_warning_output()
+
+    def tearDown(self):
+        self._free_warning_output()
 
     def _initSite(self, foo=0):
         self.root.site = Folder(id='site')
@@ -481,6 +566,25 @@ class importTypesToolTests(_TypeInfoSetup):
         self.failUnless('foo object' in tool.objectIds())
         self.failUnless('bar object' in tool.objectIds())
 
+    def test_migration(self):
+        from Products.CMFCore.exportimport.typeinfo import importTypesTool
+
+        site = self._initSite()
+        tool = site.portal_types
+
+        self.assertEqual(len(tool.objectIds()), 0)
+
+        context = DummyImportContext(site)
+        context._files['types.xml'] = self._NORMAL_TOOL_EXPORT
+        context._files['types/foo.xml'] = _FOO_EXPORT % 'foo'
+        context._files['types/bar.xml'] = _BAR_CMF21_IMPORT % 'bar'
+        importTypesTool(context)
+
+        self.assertEqual(len(tool.objectIds()), 2)
+        self.failUnless('foo' in tool.objectIds())
+        self.failUnless('bar' in tool.objectIds())
+        self.assertEqual(tool.bar.icon_expr, 'string:${portal_url}/bar.png')
+
     def test_normal_update(self):
         from Products.CMFCore.exportimport.typeinfo import importTypesTool
 
@@ -495,7 +599,7 @@ class importTypesToolTests(_TypeInfoSetup):
 
         self.assertEqual(tool.foo.title, 'Foo')
         self.assertEqual(tool.foo.content_meta_type, 'Foo Thing')
-        self.assertEqual(tool.foo.content_icon, 'foo.png')
+        self.assertEqual(tool.foo.icon_expr, 'string:${portal_url}/foo.png')
         self.assertEqual(tool.foo.immediate_view, 'foo_view')
         self.assertEqual(tool.foo._aliases,
                          {'(Default)': 'foo_view', 'view': 'foo_view'})
@@ -507,11 +611,69 @@ class importTypesToolTests(_TypeInfoSetup):
 
         self.assertEqual(tool.foo.title, 'Foo')
         self.assertEqual(tool.foo.content_meta_type, 'Foo Thing')
-        self.assertEqual(tool.foo.content_icon, 'foo.png')
+        self.assertEqual(tool.foo.icon_expr, 'string:${portal_url}/foo.png')
         self.assertEqual(tool.foo.immediate_view, 'foo_view')
         self.assertEqual(tool.foo._aliases,
                {'(Default)': 'foo_view', 'view': 'foo_view', 'spam': 'eggs'})
 
+    def test_action_remove(self):
+        from Products.CMFCore.exportimport.typeinfo import importTypesTool
+
+        site = self._initSite()
+        tool = site.portal_types
+
+        self.assertEqual(len(tool.objectIds()), 0)
+
+        context = DummyImportContext(site, False)
+
+        # Make sure removing a non-existant action doesn't fail
+        _TOOL = """\
+        <?xml version="1.0"?>
+        <object name="portal_types" meta_type="CMF Types Tool">
+         <object name="%s" meta_type="Factory-based Type Information"/>
+        </object>
+        """
+        context._files['types.xml'] = (_TOOL % 'baz').strip()
+
+        _BAZ_SETUP = """\
+        <?xml version="1.0"?>
+        <object name="%s" meta_type="Factory-based Type Information">
+         <property name="title">Baz</property>
+         <action title="View" action_id="view" category="object"
+            condition_expr="" url_expr="string:${object_url}/baz_view" 
+            icon_expr="" visible="True">
+          <permission value="View"/>
+         </action>
+         <action action_id="edit" category="object" remove="True" />
+        </object>
+        """
+        context._files['types/baz.xml'] = (_BAZ_SETUP % 'baz').strip()
+        importTypesTool(context)
+
+        self.assertEqual(len(tool.objectIds()), 1)
+        self.failUnless('baz' in tool.objectIds())
+        baz = tool['baz']
+        actions = baz.listActions()
+        self.assertEqual(len(actions), 1)
+        self.assertEqual(actions[0].title, 'View')
+
+        # Remove an already existing action
+        _BAZ_REMOVE = """\
+        <?xml version="1.0"?>
+        <object name="%s" meta_type="Factory-based Type Information">
+         <property name="title">Baz</property>
+         <action action_id="view" category="object" remove="True" />
+        </object>
+        """
+        context._files['types/baz.xml'] = (_BAZ_REMOVE % 'baz').strip()
+        importTypesTool(context)
+
+        self.assertEqual(len(tool.objectIds()), 1)
+        self.failUnless('baz' in tool.objectIds())
+        baz = tool['baz']
+        actions = baz.listActions()
+        self.assertEqual(len(actions), 0)
+
 
 def test_suite():
     return unittest.TestSuite((
