@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Foundation and Contributors.
+# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,7 +12,7 @@
 ##############################################################################
 """Workflow tool xml adapters and setup handlers.
 
-$Id: workflow.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: workflow.py 107359 2009-12-30 15:33:55Z jens $
 """
 
 from zope.component import adapts
@@ -121,7 +121,11 @@ def importWorkflowTool(context):
     """Import workflow tool and contained workflow definitions from XML files.
     """
     site = context.getSite()
-    tool = getToolByName(site, 'portal_workflow')
+    tool = getToolByName(site, 'portal_workflow', None)
+    if tool is None:
+        logger = context.getLogger('workflow')
+        logger.debug('Nothing to import.')
+        return
 
     importObjects(tool, '', context)
 
@@ -132,7 +136,7 @@ def exportWorkflowTool(context):
     tool = getToolByName(site, 'portal_workflow', None)
     if tool is None:
         logger = context.getLogger('workflow')
-        logger.info('Nothing to export.')
+        logger.debug('Nothing to export.')
         return
 
     exportObjects(tool, '', context)
