@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Foundation and Contributors.
+# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,7 +12,7 @@
 ##############################################################################
 """ CMFCore content interfaces.
 
-$Id: _content.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: _content.py 105763 2009-11-17 17:54:04Z yuppie $
 """
 
 from zope.interface import Interface
@@ -354,7 +354,7 @@ class IDublinCore(IMinimalDublinCore):
         """
 
     def Rights():
-        """ Reutrn the DCMI Rights element (resource copyright).
+        """ Return the DCMI Rights element (resource copyright).
 
         o Return a string describing the intellectual property status, if
           any, of the resource.
@@ -498,8 +498,8 @@ class IDynamicType(Interface):
         o Permission:  Public
         """
 
-    def getIcon(relative_to_portal=False):
-        """ Get the path to an object's icon.
+    def getIconURL():
+        """ Get the absolute URL of the icon for the object.
 
         o This method is used in the 'folder_contents' view to generate an
           appropriate icon for the items found in the folder.
@@ -508,14 +508,60 @@ class IDynamicType(Interface):
           return a "default" icon path (e.g., '/misc_/dtmldoc.gif', which is
           the icon used for DTML Documents).
 
-        o If 'relative_to_portal' is True, return only the portion of
-          the icon's URL which finds it "within" the portal;  otherwise,
-          return it as an absolute URL.
-
         o Permission:  Public
         """
 
 
+class ICatalogAware(Interface):
+
+    """ Interface for notifying the catalog tool.
+    """
+
+    def indexObject():
+        """ Index the object in the portal catalog.
+        """
+
+    def unindexObject():
+        """ Unindex the object from the portal catalog.
+        """
+
+    def reindexObject(idxs=[]):
+        """ Reindex the object in the portal catalog.
+
+        If idxs is present, only those indexes are reindexed. The metadata is
+        always updated.
+
+        Also update the modification date of the object, unless specific
+        indexes were requested.
+        """
+
+    def reindexObjectSecurity(skip_self=False):
+        """ Reindex security-related indexes on the object.
+
+        Recurses in the children to reindex them too.
+
+        If skip_self is True, only the children will be reindexed. This is a
+        useful optimization if the object itself has just been fully
+        reindexed, as there's no need to reindex its security twice.
+        """
+
+
+class IWorkflowAware(Interface):
+
+    """ Interface for notifying the workflow tool.
+    """
+
+    def notifyWorkflowCreated():
+        """ Notify the workflow that the object was just created.
+        """
+
+
+class IOpaqueItemManager(Interface):
+
+    """Interface for managing opaque items.
+    """
+
+
 #
 #   Folderish interfaces
 #
