@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Foundation and Contributors.
+# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,22 +12,45 @@
 ##############################################################################
 """ Unit tests for CatalogTool module.
 
-$Id: test_CatalogTool.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: test_CatalogTool.py 98305 2009-03-23 10:18:07Z miwa $
 """
 
 import unittest
-import Testing
-
-from AccessControl.SecurityManagement import getSecurityManager
-from AccessControl.SecurityManagement import newSecurityManager
-from DateTime import DateTime
 
+from Acquisition import Implicit
+from zope.interface import implements
+from zope.component import getMultiAdapter
+from Products.CMFCore.interfaces import ICatalogTool
 from Products.CMFCore.tests.base.dummy import DummyContent
-from Products.CMFCore.tests.base.dummy import DummySite
-from Products.CMFCore.tests.base.security import OmnipotentUser
-from Products.CMFCore.tests.base.security import UserWithRoles
+from Products.CMFCore.interfaces import IIndexableObject
+from Products.CMFCore.interfaces import IContentish
+
 from Products.CMFCore.tests.base.testcase import SecurityTest
 
+class FakeFolder(Implicit):
+    id = 'portal'
+
+class FakeCatalog(Implicit):
+    implements(ICatalogTool)
+    id = 'portal_catalog'
+
+class FakeWorkflowTool(Implicit):
+    id = 'portal_workflow'
+
+    def __init__(self, vars):
+        self._vars = vars
+
+    def getCatalogVariablesFor(self, ob):
+        return self._vars
+
+class CatalogDummyContent(DummyContent):
+
+    """ Dummy content that already provides IIndexableObject
+        and therefore does not need a wrapper to be registered
+    """
+
+    implements(IIndexableObject)
+    allowedRolesAndUsers = ['Manager'] # default value
 
 class IndexableObjectWrapperTests(unittest.TestCase):
 
@@ -36,17 +59,19 @@ class IndexableObjectWrapperTests(unittest.TestCase):
 
         return IndexableObjectWrapper
 
-    def _makeOne(self, *args, **kw):
-        return self._getTargetClass()(*args, **kw)
+    def _makeOne(self, vars, obj):
+        self.root = FakeFolder()
+        self.root.portal_catalog = FakeCatalog()
+        self.root.portal_workflow = FakeWorkflowTool(vars)
+        catalog = self.root.portal_catalog
+        return self._getTargetClass()(obj, catalog)
 
-    def test_z2interfaces(self):
-        from Interface.Verify import verifyClass
-        from Products.CMFCore.interfaces.portal_catalog \
-                import IndexableObjectWrapper as IIndexableObjectWrapper
+    def _makeContent(self, *args, **kw):
+        from Products.CMFCore.tests.base.dummy import DummyContent
 
-        verifyClass(IIndexableObjectWrapper, self._getTargetClass())
+        return DummyContent(*args, **kw)
 
-    def test_z3interfaces(self):
+    def test_interfaces(self):
         from zope.interface.verify import verifyClass
         from Products.CMFCore.interfaces import IIndexableObjectWrapper
 
@@ -56,24 +81,24 @@ class IndexableObjectWrapperTests(unittest.TestCase):
         # XXX This test fails when verbose security is enabled in zope.conf,
         # because the roles will then contain '_View_Permission' as well as
         # 'Manager'.
-        obj = DummyContent()
+        obj = self._makeContent()
         w = self._makeOne({}, obj)
         self.assertEqual(w.allowedRolesAndUsers(), ['Manager'])
 
     def test___str__(self):
-        obj = DummyContent('foo')
+        obj = self._makeContent('foo')
         w = self._makeOne({}, obj)
         self.assertEqual(str(w), str(obj))
 
     def test_proxied_attributes(self):
-        obj = DummyContent('foo')
+        obj = self._makeContent('foo')
         obj.title = 'Foo'
         w = self._makeOne({}, obj)
         self.assertEqual(w.getId(), 'foo')
         self.assertEqual(w.Title(), 'Foo')
 
     def test_vars(self):
-        obj = DummyContent()
+        obj = self._makeContent()
         w = self._makeOne({'bar': 1, 'baz': 2}, obj)
         self.assertEqual(w.bar, 1)
         self.assertEqual(w.baz, 2)
@@ -81,12 +106,22 @@ class IndexableObjectWrapperTests(unittest.TestCase):
     def test_provided(self):
         from Products.CMFCore.interfaces import IContentish
         from Products.CMFCore.interfaces import IIndexableObjectWrapper
+        from Products.CMFCore.interfaces import IIndexableObject
 
-        obj = DummyContent()
+        obj = self._makeContent()
         w = self._makeOne({}, obj)
         self.failUnless(IContentish.providedBy(w))
         self.failUnless(IIndexableObjectWrapper.providedBy(w))
+        self.failUnless(IIndexableObject.providedBy(w))
 
+    def test_adapts(self):
+        from zope.component import adaptedBy
+        from Products.CMFCore.interfaces import IContentish
+        from Products.CMFCore.interfaces import ICatalogTool
+
+        w = self._getTargetClass()
+        adapts =  adaptedBy(w) 
+        self.assertEqual(adapts, (IContentish, ICatalogTool))
 
 class CatalogToolTests(SecurityTest):
 
@@ -98,19 +133,10 @@ class CatalogToolTests(SecurityTest):
     def _makeOne(self, *args, **kw):
         return self._getTargetClass()(*args, **kw)
 
-    def test_z2interfaces(self):
-        from Interface.Verify import verifyClass
-        from Products.CMFCore.interfaces.portal_actions \
-                import ActionProvider as IActionProvider
-        from Products.CMFCore.interfaces.portal_catalog \
-                import portal_catalog as ICatalogTool
-        from Products.ZCatalog.IZCatalog import IZCatalog
+    def _makeContent(self, *args, **kw):
+        return CatalogDummyContent(*args, **kw)
 
-        verifyClass(IActionProvider, self._getTargetClass())
-        verifyClass(ICatalogTool, self._getTargetClass())
-        verifyClass(IZCatalog, self._getTargetClass())
-
-    def test_z3interfaces(self):
+    def test_interfaces(self):
         from zope.interface.verify import verifyClass
         from Products.CMFCore.interfaces import IActionProvider
         from Products.CMFCore.interfaces import ICatalogTool
@@ -121,10 +147,14 @@ class CatalogToolTests(SecurityTest):
         verifyClass(IZCatalog, self._getTargetClass())
 
     def loginWithRoles(self, *roles):
+        from AccessControl.SecurityManagement import newSecurityManager
+        from Products.CMFCore.tests.base.security import UserWithRoles
         user = UserWithRoles(*roles).__of__(self.root)
         newSecurityManager(None, user)
 
     def loginManager(self):
+        from AccessControl.SecurityManagement import newSecurityManager
+        from Products.CMFCore.tests.base.security import OmnipotentUser
         user = OmnipotentUser().__of__(self.root)
         newSecurityManager(None, user)
 
@@ -142,7 +172,7 @@ class CatalogToolTests(SecurityTest):
         """
         tool = self._makeOne()
         tool.addIndex('SearchableText', 'KeywordIndex')
-        dummy = DummyContent(catalog=1)
+        dummy = self._makeContent(catalog=1)
 
         tool.catalog_object( dummy, '/dummy' )
         tool.catalog_object( dummy, '/dummy', [ 'SearchableText' ] )
@@ -150,7 +180,7 @@ class CatalogToolTests(SecurityTest):
     def test_search_anonymous(self):
         catalog = self._makeOne()
         catalog.addIndex('allowedRolesAndUsers', 'KeywordIndex')
-        dummy = DummyContent(catalog=1)
+        dummy = self._makeContent(catalog=1)
         catalog.catalog_object(dummy, '/dummy')
 
         self.assertEqual(1, len(catalog._catalog.searchResults()))
@@ -159,8 +189,8 @@ class CatalogToolTests(SecurityTest):
     def test_search_member_with_valid_roles(self):
         catalog = self._makeOne()
         catalog.addIndex('allowedRolesAndUsers', 'KeywordIndex')
-        dummy = DummyContent(catalog=1)
-        dummy._View_Permission = ('Blob',)
+        dummy = self._makeContent(catalog=1)
+        dummy.allowedRolesAndUsers = ('Blob',)
         catalog.catalog_object(dummy, '/dummy')
 
         self.loginWithRoles('Blob')
@@ -171,8 +201,8 @@ class CatalogToolTests(SecurityTest):
     def test_search_member_with_valid_roles_but_proxy_roles_limit(self):
         catalog = self._makeOne()
         catalog.addIndex('allowedRolesAndUsers', 'KeywordIndex')
-        dummy = DummyContent(catalog=1)
-        dummy._View_Permission = ('Blob',)
+        dummy = self._makeContent(catalog=1)
+        dummy.allowedRolesAndUsers = ('Blob',)
         catalog.catalog_object(dummy, '/dummy')
 
         self.loginWithRoles('Blob')
@@ -184,8 +214,8 @@ class CatalogToolTests(SecurityTest):
     def test_search_member_wo_valid_roles(self):
         catalog = self._makeOne()
         catalog.addIndex('allowedRolesAndUsers', 'KeywordIndex')
-        dummy = DummyContent(catalog=1)
-        dummy._View_Permission = ('Blob',)
+        dummy = self._makeContent(catalog=1)
+        dummy.allowedRoleAndUsers = ('Blob',)
         catalog.catalog_object(dummy, '/dummy')
 
         self.loginWithRoles('Waggle')
@@ -196,8 +226,8 @@ class CatalogToolTests(SecurityTest):
     def test_search_member_wo_valid_roles_but_proxy_roles_allow(self):
         catalog = self._makeOne()
         catalog.addIndex('allowedRolesAndUsers', 'KeywordIndex')
-        dummy = DummyContent(catalog=1)
-        dummy._View_Permission = ('Blob',)
+        dummy = self._makeContent(catalog=1)
+        dummy.allowedRolesAndUsers = ('Blob',)
         catalog.catalog_object(dummy, '/dummy')
 
         self.loginWithRoles('Waggle')
@@ -207,13 +237,14 @@ class CatalogToolTests(SecurityTest):
         self.assertEqual(1, len(catalog.searchResults()))
 
     def test_search_inactive(self):
+        from DateTime.DateTime import DateTime
         catalog = self._makeOne()
         catalog.addIndex('allowedRolesAndUsers', 'KeywordIndex')
         catalog.addIndex('effective', 'DateIndex')
         catalog.addIndex('expires', 'DateIndex')
         now = DateTime()
-        dummy = DummyContent(catalog=1)
-        dummy._View_Permission = ('Blob',)
+        dummy = self._makeContent(catalog=1)
+        dummy.allowedRolesAndUsers = ('Blob',)
 
         self.loginWithRoles('Blob')
 
@@ -232,12 +263,13 @@ class CatalogToolTests(SecurityTest):
         self.assertEqual(0, len(catalog.searchResults()))
 
     def test_search_restrict_manager(self):
+        from DateTime.DateTime import DateTime
         catalog = self._makeOne()
         catalog.addIndex('allowedRolesAndUsers', 'KeywordIndex')
         catalog.addIndex('effective', 'DateIndex')
         catalog.addIndex('expires', 'DateIndex')
         now = DateTime()
-        dummy = DummyContent(catalog=1)
+        dummy = self._makeContent(catalog=1)
 
         self.loginManager()
 
@@ -266,13 +298,14 @@ class CatalogToolTests(SecurityTest):
             expires={'query': now-2, 'range': None})))
 
     def test_search_restrict_inactive(self):
+        from DateTime.DateTime import DateTime
         catalog = self._makeOne()
         catalog.addIndex('allowedRolesAndUsers', 'KeywordIndex')
         catalog.addIndex('effective', 'DateIndex')
         catalog.addIndex('expires', 'DateIndex')
         now = DateTime()
-        dummy = DummyContent(catalog=1)
-        dummy._View_Permission = ('Blob',)
+        dummy = self._makeContent(catalog=1)
+        dummy.allowedRolesAndUsers = ('Blob',)
 
         self.loginWithRoles('Blob')
 
@@ -299,13 +332,14 @@ class CatalogToolTests(SecurityTest):
             expires={'query': now-2, 'range': None})))
 
     def test_search_restrict_visible(self):
+        from DateTime.DateTime import DateTime
         catalog = self._makeOne()
         catalog.addIndex('allowedRolesAndUsers', 'KeywordIndex')
         catalog.addIndex('effective', 'DateIndex')
         catalog.addIndex('expires', 'DateIndex')
         now = DateTime()
-        dummy = DummyContent(catalog=1)
-        dummy._View_Permission = ('Blob',)
+        dummy = self._makeContent(catalog=1)
+        dummy.allowedRolesAndUsers = ('Blob',)
 
         self.loginWithRoles('Blob')
 
@@ -405,8 +439,9 @@ class CatalogToolTests(SecurityTest):
         self.assertEqual(kw, {'expires': {'query': (5,7), 'range': 'min:max'}})
 
     def test_refreshCatalog(self):
+        from Products.CMFCore.tests.base.dummy import DummySite
         site = DummySite('site').__of__(self.root)
-        site._setObject('dummy', DummyContent(catalog=1))
+        site._setObject('dummy', self._makeContent(catalog=1))
         site._setObject('portal_catalog', self._makeOne())
         ctool = site.portal_catalog
         ctool.catalog_object(site.dummy, '/dummy')
@@ -420,6 +455,7 @@ class CatalogToolTests(SecurityTest):
 
     def test_listAllowedRolesAndUsers_proxyroles(self):
         # https://bugs.launchpad.net/zope-cmf/+bug/161729
+        from AccessControl import getSecurityManager
         catalog = self._makeOne()
         self.loginWithRoles('Blob')
         user = getSecurityManager().getUser()
@@ -449,6 +485,40 @@ class CatalogToolTests(SecurityTest):
         self.failUnless('Blob' in arus)
         self.failUnless('user:%s' % user.getId() in arus)       
 
+    def test_wrapping1(self):
+        # DummyContent implements IIndexableObject
+        # so should be indexed
+        dummy = self._makeContent(catalog=1)
+        ctool = self._makeOne()
+        ctool.catalog_object(dummy, '/dummy')
+        self.assertEqual(1, len(ctool._catalog.searchResults()))
+
+    def test_wrapping2(self):
+        # DummyContent does not implement IIndexableObject
+        # no wrapper registered - should fall back to using
+        # wrapper class directly
+        dummy = DummyContent(catalog=1)
+        ctool = self._makeOne()
+        ctool.catalog_object(dummy, '/dummy')
+        self.assertEqual(1, len(ctool._catalog.searchResults()))
+
+    def test_wrapping3(self):
+        # DummyContent does not implement IIndexableObject
+        # wrapper registered - should look this up
+
+        def FakeWrapper(object, catalog):
+            return object
+
+        from zope.component import getSiteManager
+        self.sm = getSiteManager()
+        self.sm.registerAdapter( FakeWrapper
+                               , (IContentish, ICatalogTool)
+                               , IIndexableObject )
+
+        dummy =DummyContent(catalog=1)
+        ctool = self._makeOne()
+        ctool.catalog_object(dummy, '/dummy')
+        self.assertEqual(1, len(ctool._catalog.searchResults()))
 
 def test_suite():
     return unittest.TestSuite((
