@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Foundation and Contributors.
+# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,7 +12,7 @@
 ##############################################################################
 """ Utility functions.
 
-$Id: utils.py 110655 2010-04-08 15:38:57Z tseaver $
+$Id: utils.py 100495 2009-05-27 22:59:18Z mauritsvanrees $
 """
 
 import re
@@ -22,27 +22,27 @@ from os.path import abspath
 from warnings import warn
 import sys
 
-from AccessControl import ClassSecurityInfo
-from AccessControl import getSecurityManager
-from AccessControl import ModuleSecurityInfo
 from AccessControl.Permission import Permission
 from AccessControl.PermissionRole import rolesForPermissionOn
 from AccessControl.Role import gather_permissions
+from AccessControl.SecurityInfo import ClassSecurityInfo
+from AccessControl.SecurityInfo import ModuleSecurityInfo
+from AccessControl.SecurityManagement import getSecurityManager
 from Acquisition.interfaces import IAcquirer
 from Acquisition import aq_get
 from Acquisition import aq_inner
 from Acquisition import aq_parent
 from Acquisition import Implicit
-from DateTime import DateTime
+from App.class_init import InitializeClass
+from App.Common import package_home
+from App.Dialogs import MessageDialog
+from App.ImageFile import ImageFile
+from App.special_dtml import HTMLFile
+from DateTime.DateTime import DateTime
 from ExtensionClass import Base
-from Globals import HTMLFile
-from Globals import ImageFile
-from Globals import InitializeClass
-from Globals import MessageDialog
-from Globals import package_home
-from Globals import UNIQUE
 from OFS.misc_ import misc_ as misc_images
 from OFS.misc_ import Misc_ as MiscImage
+from OFS.ObjectManager import UNIQUE
 from OFS.PropertyManager import PropertyManager
 from OFS.PropertySheets import PropertySheets
 from OFS.SimpleItem import SimpleItem
@@ -53,8 +53,8 @@ from zope.component.interfaces import ComponentLookupError
 from zope.dottedname.resolve import resolve as resolve_dotted_name
 from zope.i18nmessageid import MessageFactory
 
-from exceptions import AccessControl_Unauthorized
-from exceptions import NotFound
+from Products.CMFCore.exceptions import AccessControl_Unauthorized
+from Products.CMFCore.exceptions import NotFound
 
 SUBTEMPLATE = '__SUBTEMPLATE__'
 
@@ -74,6 +74,8 @@ _tool_interface_registry = {}
 security.declarePrivate('registerToolInterface')
 def registerToolInterface(tool_id, tool_interface):
     """ Register a tool ID for an interface
+
+    This method can go away when getToolByName is going away (CMF 2.3).
     """
     global  _tool_interface_registry
     _tool_interface_registry[tool_id] = tool_interface
@@ -503,7 +505,10 @@ class UniqueObject (ImmutableId):
 
     """ Base class for objects which cannot be "overridden" / shadowed.
     """
-    __replaceable__ = UNIQUE
+    def _getUNIQUE(self):
+        return UNIQUE
+
+    __replaceable__ = property(_getUNIQUE,)
 
 
 class SimpleItemWithProperties (PropertyManager, SimpleItem):
@@ -632,12 +637,14 @@ class ContentInit:
                 , permission=None
                 , extra_constructors=()
                 , fti=()
+                , visibility='Global'
                 ):
         # BBB: fti argument is ignored
         self.meta_type = meta_type
         self.content_types = content_types
         self.permission = permission
         self.extra_constructors = extra_constructors
+        self.visibility = visibility
 
     def initialize(self, context):
         # Add only one meta type to the folder add list.
@@ -650,6 +657,7 @@ class ContentInit:
                                , manage_addContent
                                , self ) + self.extra_constructors
             , permission = self.permission
+            , visibility = self.visibility
             )
 
         for ct in self.content_types:
